@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- * libmatewnck based pager applet.
+ * libwnck based pager applet.
  * (C) 2001 Alexander Larsson
  * (C) 2001 Red Hat, Inc
  *
@@ -21,9 +21,12 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libmatewnck/libmatewnck.h>
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE
+#include <libwnck/libwnck.h>
 #include <gio/gio.h>
 
+#include <libmate-desktop/mate-gsettings.h>
+
 #include "workspace-switcher.h"
 
 #include "wncklet.h"
@@ -53,7 +56,7 @@ typedef struct {
 
 	GtkWidget* pager;
 
-	MatewnckScreen* screen;
+	WnckScreen* screen;
 	PagerWM wm;
 
 	/* Properties: */
@@ -73,7 +76,7 @@ typedef struct {
 
 	GtkOrientation orientation;
 	int n_rows;				/* for vertical layout this is cols */
-	MatewnckPagerDisplayMode display_mode;
+	WnckPagerDisplayMode display_mode;
 	gboolean display_all;
 	gboolean wrap_workspaces;
 
@@ -86,14 +89,14 @@ static void display_about_dialog(GtkAction* action, PagerData* pager);
 
 static void pager_update(PagerData* pager)
 {
-	matewnck_pager_set_orientation(MATEWNCK_PAGER(pager->pager), pager->orientation);
-	matewnck_pager_set_n_rows(MATEWNCK_PAGER(pager->pager), pager->n_rows);
-	matewnck_pager_set_show_all(MATEWNCK_PAGER(pager->pager), pager->display_all);
+	wnck_pager_set_orientation(WNCK_PAGER(pager->pager), pager->orientation);
+	wnck_pager_set_n_rows(WNCK_PAGER(pager->pager), pager->n_rows);
+	wnck_pager_set_show_all(WNCK_PAGER(pager->pager), pager->display_all);
 
 	if (pager->wm == PAGER_WM_MARCO)
-		matewnck_pager_set_display_mode(MATEWNCK_PAGER(pager->pager), pager->display_mode);
+		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), pager->display_mode);
 	else
-		matewnck_pager_set_display_mode(MATEWNCK_PAGER(pager->pager), MATEWNCK_PAGER_DISPLAY_CONTENT);
+		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), WNCK_PAGER_DISPLAY_CONTENT);
 }
 
 static void update_properties_for_wm(PagerData* pager)
@@ -132,11 +135,11 @@ static void update_properties_for_wm(PagerData* pager)
 		gtk_window_reshow_with_initial_size(GTK_WINDOW(pager->properties_dialog));
 }
 
-static void window_manager_changed(MatewnckScreen* screen, PagerData* pager)
+static void window_manager_changed(WnckScreen* screen, PagerData* pager)
 {
 	const char *wm_name;
 
-	wm_name = matewnck_screen_get_window_manager_name(screen);
+	wm_name = wnck_screen_get_window_manager_name(screen);
 
 	if (!wm_name)
 		pager->wm = PAGER_WM_UNKNOWN;
@@ -192,7 +195,11 @@ static void applet_change_orient(MatePanelApplet* applet, MatePanelAppletOrient
 		gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, cairo_pattern_t *pattern, PagerData* pager)
+#else
 static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, GdkPixmap* pixmap, PagerData* pager)
+#endif
 {
         /* taken from the TrashApplet */
         GtkRcStyle *rc_style;
@@ -204,6 +211,10 @@ static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBac
         gtk_widget_modify_style (GTK_WIDGET (pager->pager), rc_style);
         g_object_unref (rc_style);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+		type == PANEL_NO_BACKGROUND ? GTK_SHADOW_NONE : GTK_SHADOW_IN);
+#else
 	switch (type)
 	{
                 case PANEL_COLOR_BACKGROUND:
@@ -223,6 +234,7 @@ static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBac
                 default:
                         break;
 	}
+#endif
 }
 
 static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, PagerData* pager)
@@ -236,8 +248,8 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 	if (event->type != GDK_SCROLL)
 		return FALSE;
 
-	index = matewnck_workspace_get_number(matewnck_screen_get_active_workspace(pager->screen));
-	n_workspaces = matewnck_screen_get_workspace_count(pager->screen);
+	index = wnck_workspace_get_number(wnck_screen_get_active_workspace(pager->screen));
+	n_workspaces = wnck_screen_get_workspace_count(pager->screen);
 	n_columns = n_workspaces / pager->n_rows;
 
 	if (n_workspaces % pager->n_rows != 0)
@@ -324,7 +336,7 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 			break;
 	}
 
-	matewnck_workspace_activate(matewnck_screen_get_workspace(pager->screen, index), event->time);
+	wnck_workspace_activate(wnck_screen_get_workspace(pager->screen, index), event->time);
 
 	return TRUE;
 }
@@ -389,11 +401,11 @@ static void display_workspace_names_changed(GSettings* settings, gchar* key, Pag
 
 	if (value)
 	{
-		pager->display_mode = MATEWNCK_PAGER_DISPLAY_NAME;
+		pager->display_mode = WNCK_PAGER_DISPLAY_NAME;
 	}
 	else
 	{
-		pager->display_mode = MATEWNCK_PAGER_DISPLAY_CONTENT;
+		pager->display_mode = WNCK_PAGER_DISPLAY_CONTENT;
 	}
 
 	pager_update(pager);
@@ -495,11 +507,11 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 
 	if (display_names)
 	{
-		pager->display_mode = MATEWNCK_PAGER_DISPLAY_NAME;
+		pager->display_mode = WNCK_PAGER_DISPLAY_NAME;
 	}
 	else
 	{
-		pager->display_mode = MATEWNCK_PAGER_DISPLAY_CONTENT;
+		pager->display_mode = WNCK_PAGER_DISPLAY_CONTENT;
 	}
 
 	pager->display_all = g_settings_get_boolean(pager->settings, "display-all-workspaces");
@@ -517,10 +529,18 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 			break;
 	}
 
-	pager->pager = matewnck_pager_new(NULL);
+#ifdef WNCK_CHECK_VERSION
+#if WNCK_CHECK_VERSION (2, 91, 6)
+	pager->pager = wnck_pager_new();
+#else
+	pager->pager = wnck_pager_new(NULL);
+#endif
+#else
+	pager->pager = wnck_pager_new(NULL);
+#endif
 	pager->screen = NULL;
 	pager->wm = PAGER_WM_UNKNOWN;
-	matewnck_pager_set_shadow_type(MATEWNCK_PAGER(pager->pager), GTK_SHADOW_IN);
+	wnck_pager_set_shadow_type(WNCK_PAGER(pager->pager), GTK_SHADOW_IN);
 
 	g_signal_connect(G_OBJECT(pager->pager), "destroy", G_CALLBACK(destroy_pager), pager);
 
@@ -560,7 +580,7 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 
 static void display_help_dialog(GtkAction* action, PagerData* pager)
 {
-	wncklet_display_help(pager->applet, "user-guide", "overview-workspaces", WORKSPACE_SWITCHER_ICON);
+	wncklet_display_help(pager->applet, "mate-user-guide", "overview-workspaces", WORKSPACE_SWITCHER_ICON);
 }
 
 static void display_about_dialog(GtkAction* action, PagerData* pager)
@@ -620,10 +640,10 @@ static void num_rows_value_changed(GtkSpinButton* button, PagerData* pager)
 static void update_workspaces_model(PagerData* pager)
 {
 	int nr_ws, i;
-	MatewnckWorkspace* workspace;
+	WnckWorkspace* workspace;
 	GtkTreeIter iter;
 
-	nr_ws = matewnck_screen_get_workspace_count(pager->screen);
+	nr_ws = wnck_screen_get_workspace_count(pager->screen);
 
 	if (pager->properties_dialog)
 	{
@@ -634,47 +654,49 @@ static void update_workspaces_model(PagerData* pager)
 
 		for (i = 0; i < nr_ws; i++)
 		{
-			workspace = matewnck_screen_get_workspace(pager->screen, i);
+			workspace = wnck_screen_get_workspace(pager->screen, i);
 			gtk_list_store_append(pager->workspaces_store, &iter);
-			gtk_list_store_set(pager->workspaces_store, &iter, 0, matewnck_workspace_get_name(workspace), -1);
+			gtk_list_store_set(pager->workspaces_store, &iter, 0, wnck_workspace_get_name(workspace), -1);
 		}
 	}
 }
 
-static void workspace_renamed(MatewnckWorkspace* space, PagerData* pager)
+static void workspace_renamed(WnckWorkspace* space, PagerData* pager)
 {
 	int i;
 	GtkTreeIter iter;
 
-	i = matewnck_workspace_get_number(space);
+	i = wnck_workspace_get_number(space);
 
 	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(pager->workspaces_store), &iter, NULL, i))
-		gtk_list_store_set(pager->workspaces_store, &iter, 0, matewnck_workspace_get_name(space), -1);
+		gtk_list_store_set(pager->workspaces_store, &iter, 0, wnck_workspace_get_name(space), -1);
 }
 
-static void workspace_created(MatewnckScreen* screen, MatewnckWorkspace* space, PagerData* pager)
+static void workspace_created(WnckScreen* screen, WnckWorkspace* space, PagerData* pager)
 {
-	g_return_if_fail(MATEWNCK_IS_SCREEN(screen));
+	g_return_if_fail(WNCK_IS_SCREEN(screen));
 
 	update_workspaces_model(pager);
 
 	wncklet_connect_while_alive(space, "name_changed", G_CALLBACK(workspace_renamed), pager, pager->properties_dialog);
 }
 
-static void workspace_destroyed(MatewnckScreen* screen, MatewnckWorkspace* space, PagerData* pager)
+static void workspace_destroyed(WnckScreen* screen, WnckWorkspace* space, PagerData* pager)
 {
-	g_return_if_fail(MATEWNCK_IS_SCREEN(screen));
+	g_return_if_fail(WNCK_IS_SCREEN(screen));
 	update_workspaces_model(pager);
 }
 
 static void num_workspaces_value_changed(GtkSpinButton* button, PagerData* pager)
 {
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	/* Slow down a bit after the first change, since it's moving really to
 	 * fast. See bug #336731 for background.
 	 * FIXME: remove this if bug 410520 gets fixed. */
 	button->timer_step = 0.2;
+#endif
 
-	matewnck_screen_change_workspace_count(pager->screen, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pager->num_workspaces_spin)));
+	wnck_screen_change_workspace_count(pager->screen, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pager->num_workspaces_spin)));
 }
 
 static gboolean workspaces_tree_focused_out(GtkTreeView* treeview, GdkEventFocus* event, PagerData* pager)
@@ -689,18 +711,18 @@ static gboolean workspaces_tree_focused_out(GtkTreeView* treeview, GdkEventFocus
 static void workspace_name_edited(GtkCellRendererText* cell_renderer_text, const gchar* path, const gchar* new_text, PagerData* pager)
 {
 	const gint* indices;
-	MatewnckWorkspace* workspace;
+	WnckWorkspace* workspace;
 	GtkTreePath* p;
 
 	p = gtk_tree_path_new_from_string(path);
 	indices = gtk_tree_path_get_indices(p);
-	workspace = matewnck_screen_get_workspace(pager->screen, indices[0]);
+	workspace = wnck_screen_get_workspace(pager->screen, indices[0]);
 
 	if (workspace != NULL)
 	{
 		gchar* temp_name = g_strdup(new_text);
 
-		matewnck_workspace_change_name(workspace, g_strstrip(temp_name));
+		wnck_workspace_change_name(workspace, g_strstrip(temp_name));
 
 		g_free(temp_name);
 	}
@@ -738,7 +760,7 @@ static gboolean delete_event(GtkWidget* widget, gpointer data)
 static void response_cb(GtkWidget* widget, int id, PagerData* pager)
 {
 	if (id == GTK_RESPONSE_HELP)
-		wncklet_display_help(widget, "user-guide", "overview-workspaces", WORKSPACE_SWITCHER_ICON);
+		wncklet_display_help(widget, "mate-user-guide", "overview-workspaces", WORKSPACE_SWITCHER_ICON);
 	else
 		gtk_widget_destroy(widget);
 }
@@ -747,6 +769,10 @@ static void close_dialog(GtkWidget* button, gpointer data)
 {
 	PagerData* pager = data;
 	GtkTreeViewColumn* col;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCellArea *area;
+	GtkCellEditable *edit_widget;
+#endif
 
 	/* This is a hack. The "editable" signal for GtkCellRenderer is emitted
 	only on button press or focus cycle. Hence when the user changes the
@@ -757,8 +783,15 @@ static void close_dialog(GtkWidget* button, gpointer data)
 
 	col = gtk_tree_view_get_column(GTK_TREE_VIEW(pager->workspaces_tree), 0);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	area = gtk_cell_layout_get_area (GTK_CELL_LAYOUT (col));
+	edit_widget = gtk_cell_area_get_edit_widget (area);
+	if (edit_widget)
+		gtk_cell_editable_editing_done (edit_widget);
+#else
 	if (col->editable_widget != NULL && GTK_IS_CELL_EDITABLE(col->editable_widget))
 		gtk_cell_editable_editing_done(col->editable_widget);
+#endif
 
 	gtk_widget_destroy(pager->properties_dialog);
 }
@@ -770,7 +803,7 @@ setup_sensitivity(PagerData* pager, GtkBuilder* builder, const char* wid1, const
 {
 	GtkWidget* w;
 
-	if (g_settings_is_writable(settings, key))
+	if ((settings != NULL) && g_settings_is_writable(settings, key))
 	{
 		return;
 	}
@@ -803,11 +836,13 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	GtkTreeViewColumn* column;
 	GtkCellRenderer* cell;
 	int nr_ws, i;
-	GSettings *marco_general_settings;
-	GSettings *marco_workspaces_settings;
+	GSettings *marco_general_settings = NULL;
+	GSettings *marco_workspaces_settings = NULL;
 
-	marco_general_settings = g_settings_new (MARCO_GENERAL_SCHEMA);
-	marco_workspaces_settings = g_settings_new (MARCO_WORSKACES_SCHEMA);
+	if (mate_gsettings_schema_exists(MARCO_GENERAL_SCHEMA))
+		marco_general_settings = g_settings_new (MARCO_GENERAL_SCHEMA);
+	if (mate_gsettings_schema_exists(MARCO_WORSKACES_SCHEMA))
+		marco_workspaces_settings = g_settings_new (MARCO_WORSKACES_SCHEMA);
 
 	pager->workspaces_frame = WID("workspaces_frame");
 	pager->workspace_names_label = WID("workspace_names_label");
@@ -833,8 +868,10 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	pager->workspaces_tree = WID("workspaces_tree_view");
 	setup_sensitivity(pager, builder, "workspaces_tree_view", NULL, NULL, marco_workspaces_settings, WORKSPACE_NAME /* key */);
 
-	g_object_unref (marco_general_settings);
-	g_object_unref (marco_workspaces_settings);
+	if (marco_general_settings != NULL)
+		g_object_unref (marco_general_settings);
+	if (marco_workspaces_settings != NULL)
+		g_object_unref (marco_workspaces_settings);
 
 
 	/* Wrap workspaces: */
@@ -850,7 +887,7 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 
 	g_signal_connect(G_OBJECT(pager->display_workspaces_toggle), "toggled", (GCallback) display_workspace_names_toggled, pager);
 
-	if (pager->display_mode == MATEWNCK_PAGER_DISPLAY_NAME)
+	if (pager->display_mode == WNCK_PAGER_DISPLAY_NAME)
 	{
 		value = TRUE;
 	}
@@ -891,7 +928,7 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 
 	g_signal_connect(WID("done_button"), "clicked", (GCallback) close_dialog, pager);
 
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_workspaces_spin), matewnck_screen_get_workspace_count(pager->screen));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_workspaces_spin), wnck_screen_get_workspace_count(pager->screen));
 	g_signal_connect(G_OBJECT(pager->num_workspaces_spin), "value_changed", (GCallback) num_workspaces_value_changed, pager);
 
 	wncklet_connect_while_alive(pager->screen, "workspace_created", G_CALLBACK(workspace_created), pager, pager->properties_dialog);
@@ -911,11 +948,11 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(pager->workspaces_tree), column);
 	g_signal_connect(cell, "edited", (GCallback) workspace_name_edited, pager);
 
-	nr_ws = matewnck_screen_get_workspace_count(pager->screen);
+	nr_ws = wnck_screen_get_workspace_count(pager->screen);
 
 	for (i = 0; i < nr_ws; i++)
 	{
-		wncklet_connect_while_alive(G_OBJECT(matewnck_screen_get_workspace(pager->screen, i)), "name_changed", G_CALLBACK(workspace_renamed), pager, pager->properties_dialog);
+		wncklet_connect_while_alive(G_OBJECT(wnck_screen_get_workspace(pager->screen, i)), "name_changed", G_CALLBACK(workspace_renamed), pager, pager->properties_dialog);
 	}
 
 	update_properties_for_wm(pager);
