@@ -180,7 +180,8 @@ mate_panel_applet_locked_change_notify (GSettings *settings,
 	AppletInfo  *info;
 	PanelWidget *panel_widget;
 
-	g_assert (applet != NULL);
+	if (applet == NULL || !GTK_IS_WIDGET (applet))
+		return;
 
 	info = (AppletInfo  *) g_object_get_data (G_OBJECT (applet),
 						  "applet_info");
@@ -254,7 +255,7 @@ applet_callback_callback (GtkWidget      *widget,
 			panel_properties_dialog_present (drawer->toplevel);
 		} else if (strcmp (menu->name, "help") == 0) {
 			panel_show_help (screen,
-					 "user-guide", "gospanel-18", NULL);
+					 "mate-user-guide", "gospanel-18", NULL);
 		}
 		break;
 	case PANEL_OBJECT_MENU:
@@ -610,7 +611,11 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 
 	screen = gtk_widget_get_screen (applet);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
+#else
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+#endif
 
 	gdk_window_get_origin (gtk_widget_get_window (applet), &menu_x, &menu_y);
 	gtk_widget_get_pointer (applet, &pointer_x, &pointer_y);
