@@ -54,7 +54,7 @@ panel_context_menu_show_help (GtkWidget *w,
 			      gpointer data)
 {
 	panel_show_help (gtk_widget_get_screen (w),
-			 "user-guide", "gospanel-1", NULL);
+			 "mate-user-guide", "gospanel-1", NULL);
 }
 
 static gboolean
@@ -65,12 +65,15 @@ panel_context_menu_check_for_screen (GtkWidget *w,
 	static int times = 0;
 	if (ev->type != GDK_KEY_PRESS)
 		return FALSE;
-	if (ev->key.keyval == GDK_f ||
-	    ev->key.keyval == GDK_F) {
+	if (ev->key.keyval == GDK_KEY_f ||
+	    ev->key.keyval == GDK_KEY_F) {
 		times++;
 		if (times == 3) {
 			times = 0;
+#if !GTK_CHECK_VERSION (3, 0, 0)
+			/* FIXME re-add once GTK3 support is fixed */
 			start_screen_check ();
+#endif
 		}
 	}
 	return FALSE;
