@@ -47,6 +47,7 @@
 #include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
 #include "panel-icon-names.h"
+#include "panel-schemas.h"
 
 G_DEFINE_TYPE (PanelMenuBar, panel_menu_bar, GTK_TYPE_MENU_BAR)
 
@@ -61,6 +62,8 @@ struct _PanelMenuBarPrivate {
 	GtkWidget* places_item;
 	GtkWidget* desktop_item;
 
+	GSettings* settings;
+
 	PanelOrientation orientation;
 };
 
@@ -107,21 +110,61 @@ static void panel_menu_bar_setup_tooltip(PanelMenuBar* menubar)
 	g_signal_connect(GTK_MENU_SHELL (menubar), "deactivate", G_CALLBACK (panel_menu_bar_reinit_tooltip), menubar);
 }
 
-static void panel_menu_bar_init(PanelMenuBar* menubar)
+static void panel_menu_bar_update_visibility (GSettings* settings, gchar* key, PanelMenuBar* menubar)
 {
 	GtkWidget* image;
+	gchar *str;
+
+	if (!GTK_IS_WIDGET (menubar))
+		return;
+
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->applications_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_APPLICATIONS_KEY));
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->places_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_PLACES_KEY));
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->desktop_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_DESKTOP_KEY));
+
+	if (g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_ICON_KEY))
+	{
+		str = g_settings_get_string (settings, PANEL_MENU_BAR_ICON_NAME_KEY);
+		if (str != NULL && str[0] != 0)
+			image = gtk_image_new_from_icon_name(str, panel_menu_bar_icon_get_size());
+		else
+			image = gtk_image_new_from_icon_name(PANEL_ICON_MAIN_MENU, panel_menu_bar_icon_get_size());
+		g_free (str);
+	}
+	else
+		image = NULL;
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menubar->priv->applications_item), image);
+}
+
+static void panel_menu_bar_init(PanelMenuBar* menubar)
+{
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCssProvider *provider;
+#endif
 
 	menubar->priv = PANEL_MENU_BAR_GET_PRIVATE(menubar);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider,
+		"PanelMenuBar {\n"
+		" border-width: 0px;\n"
+		"}",
+		-1, NULL);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (menubar)),
+		GTK_STYLE_PROVIDER (provider),
+		GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	g_object_unref (provider);
+#endif
+
 	menubar->priv->info = NULL;
 
+	menubar->priv->settings = g_settings_new (PANEL_MENU_BAR_SCHEMA);
 
 	menubar->priv->applications_menu = create_applications_menu("mate-applications.menu", NULL, TRUE);
 
 	menubar->priv->applications_item = panel_image_menu_item_new();
 	gtk_menu_item_set_label(GTK_MENU_ITEM(menubar->priv->applications_item), _("Applications"));
-	image = gtk_image_new_from_icon_name(PANEL_ICON_MAIN_MENU, panel_menu_bar_icon_get_size());
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menubar->priv->applications_item), image);
 
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menubar->priv->applications_item), menubar->priv->applications_menu);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menubar->priv->applications_item);
@@ -134,6 +177,9 @@ static void panel_menu_bar_init(PanelMenuBar* menubar)
 
 	panel_menu_bar_setup_tooltip(menubar);
 
+	panel_menu_bar_update_visibility(menubar->priv->settings, NULL, menubar);
+	g_signal_connect(menubar->priv->settings, "changed", G_CALLBACK (panel_menu_bar_update_visibility), menubar);
+
 	panel_menu_bar_update_text_gravity(menubar);
 	g_signal_connect(menubar, "screen-changed", G_CALLBACK(panel_menu_bar_update_text_gravity), NULL);
 }
@@ -233,6 +279,19 @@ static void panel_menu_bar_size_allocate(GtkWidget* widget, GtkAllocation* alloc
 	panel_menu_bar_change_background(PANEL_MENU_BAR(widget));
 }
 
+static void panel_menu_bar_finalize (GObject* object)
+{
+	PanelMenuBar* menubar;
+
+	menubar = PANEL_MENU_BAR (object);
+
+	if (menubar->priv->settings != NULL)
+	{
+		g_object_unref (menubar->priv->settings);
+		menubar->priv->settings = NULL;
+	}
+}
+
 static void panel_menu_bar_class_init(PanelMenuBarClass* klass)
 {
 	GObjectClass* gobject_class = (GObjectClass*) klass;
@@ -240,6 +299,7 @@ static void panel_menu_bar_class_init(PanelMenuBarClass* klass)
 
 	gobject_class->get_property = panel_menu_bar_get_property;
 	gobject_class->set_property = panel_menu_bar_set_property;
+	gobject_class->finalize = panel_menu_bar_finalize;
 
 	widget_class->parent_set = panel_menu_bar_parent_set;
 	widget_class->size_allocate = panel_menu_bar_size_allocate;
@@ -248,6 +308,7 @@ static void panel_menu_bar_class_init(PanelMenuBarClass* klass)
 
 	g_object_class_install_property(gobject_class, PROP_ORIENTATION, g_param_spec_enum("orientation", "Orientation", "The PanelMenuBar orientation", PANEL_TYPE_ORIENTATION, PANEL_ORIENTATION_TOP, G_PARAM_READWRITE));
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	gtk_rc_parse_string (
 		"style \"panel-menubar-style\"\n"
 		"{\n"
@@ -255,15 +316,40 @@ static void panel_menu_bar_class_init(PanelMenuBarClass* klass)
 		"  GtkMenuBar::internal-padding = 0\n"
 		"}\n"
 		"class \"PanelMenuBar\" style \"panel-menubar-style\"");
+#endif
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean panel_menu_bar_on_draw(GtkWidget* widget, cairo_t* cr, gpointer data)
+#else
 static gboolean panel_menu_bar_on_expose(GtkWidget* widget, GdkEventExpose* event, gpointer data)
+#endif
 {
 	PanelMenuBar* menubar = data;
 
 	if (gtk_widget_has_focus(GTK_WIDGET(menubar)))
 	{
-		gtk_paint_focus(gtk_widget_get_style(widget), gtk_widget_get_window(widget), gtk_widget_get_state(GTK_WIDGET(menubar)), NULL, widget, "menubar-applet", 0, 0, -1, -1);
+		gtk_paint_focus(gtk_widget_get_style(widget),
+#if GTK_CHECK_VERSION (3, 0, 0)
+						cr,
+#else
+						gtk_widget_get_window(widget),
+#endif
+						gtk_widget_get_state(GTK_WIDGET(menubar)),
+#if !GTK_CHECK_VERSION (3, 0, 0)
+						NULL,
+#endif
+						widget,
+						"menubar-applet",
+						0,
+						0,
+#if GTK_CHECK_VERSION (3, 0, 0)
+						gtk_widget_get_allocated_width (widget),
+						gtk_widget_get_allocated_height (widget));
+#else
+						-1,
+						-1);
+#endif
 	}
 
 	return FALSE;
@@ -295,7 +381,11 @@ static void panel_menu_bar_load(PanelWidget* panel, gboolean locked, int positio
 
 	g_signal_connect_after(menubar, "focus-in-event", G_CALLBACK(gtk_widget_queue_draw), menubar);
 	g_signal_connect_after(menubar, "focus-out-event", G_CALLBACK(gtk_widget_queue_draw), menubar);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_signal_connect_after(menubar, "draw", G_CALLBACK(panel_menu_bar_on_draw), menubar);
+#else
 	g_signal_connect_after(menubar, "expose-event", G_CALLBACK(panel_menu_bar_on_expose), menubar);
+#endif
 
 	gtk_widget_set_can_focus(GTK_WIDGET(menubar), TRUE);
 
@@ -327,7 +417,7 @@ void panel_menu_bar_invoke_menu(PanelMenuBar* menubar, const char* callback_name
 
 	if (!strcmp(callback_name, "help"))
 	{
-		panel_show_help(screen, "user-guide", "menubar", NULL);
+		panel_show_help(screen, "mate-user-guide", "menubar", NULL);
 
 	}
 	else if (!strcmp(callback_name, "edit"))
@@ -362,6 +452,7 @@ void panel_menu_bar_popup_menu(PanelMenuBar* menubar, guint32 activate_time)
 	 */
 	menu_shell = GTK_MENU_SHELL(menubar);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	if (!menu_shell->active)
 	{
 		gtk_grab_add(GTK_WIDGET(menu_shell));
@@ -369,6 +460,7 @@ void panel_menu_bar_popup_menu(PanelMenuBar* menubar, guint32 activate_time)
 		menu_shell->have_grab = TRUE;
 		menu_shell->active = TRUE;
 	}
+#endif
 
 	gtk_menu_shell_select_item(menu_shell, gtk_menu_get_attach_widget(menu));
 }
