@@ -26,7 +26,11 @@
 
 #include <stdlib.h>
 
+#include <gtk/gtk.h>
 #include <gdk/gdk.h>
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <gdk/gdkx.h>
+#endif
 
 #include <libegg/eggsmclient.h>
 
@@ -76,5 +80,9 @@ panel_session_init (void)
 
 	/* We don't want the WM to try and save/restore our
 	 * window position */
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_x11_set_sm_client_id (NULL);
+#else
 	gdk_set_sm_client_id (NULL);
+#endif
 }
