@@ -9,8 +9,12 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <gtk/gtkx.h> /* for GTK_IS_SOCKET */
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 
 #include <libpanel-util/panel-list.h>
 
@@ -59,6 +63,14 @@ static guint panel_widget_signals [LAST_SIGNAL] = {0};
 static gboolean mate_panel_applet_in_drag = FALSE;
 static GtkWidget *saved_focus_widget = NULL;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void panel_widget_get_preferred_width (GtkWidget *widget,
+					 gint *minimum_width,
+					 gint *natural_width);
+static void panel_widget_get_preferred_height (GtkWidget *widget,
+					 gint *minimum_height,
+					 gint *natural_height);
+#endif
 static void panel_widget_size_request   (GtkWidget        *widget,
 					 GtkRequisition   *requisition);
 static void panel_widget_size_allocate  (GtkWidget        *widget,
@@ -67,14 +79,24 @@ static void panel_widget_cadd           (GtkContainer     *container,
 					 GtkWidget        *widget);
 static void panel_widget_cremove        (GtkContainer     *container,
 					 GtkWidget        *widget);
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void panel_widget_dispose        (GObject *obj);
+#else
 static void panel_widget_destroy        (GtkObject        *obj);
+#endif
 static void panel_widget_finalize       (GObject          *obj);
 static void panel_widget_realize        (GtkWidget        *widget);
 static void panel_widget_unrealize      (GtkWidget        *panel);
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void panel_widget_state_flags_changed (GtkWidget *widget,
+					     GtkStateFlags previous_state);
+static void panel_widget_style_updated (GtkWidget *widget);
+#else
 static void panel_widget_state_changed  (GtkWidget        *widget,
 					 GtkStateType      previous_state);
 static void panel_widget_style_set      (GtkWidget        *widget,
 					 GtkStyle         *previous_style);
+#endif
 
 static void panel_widget_background_changed (PanelBackground *background,
 					     PanelWidget     *panel);
@@ -111,6 +133,11 @@ static void
 emit_applet_moved (PanelWidget *panel_widget,
 		   AppletData  *applet)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	/* we always want to queue a draw after moving, so do it here instead
+	* of after the signal emission in all callers */
+	gtk_widget_queue_draw (applet->applet);
+#endif
 	g_signal_emit (panel_widget,
 		       panel_widget_signals [APPLET_MOVE_SIGNAL], 0,
 		       applet->applet);
@@ -274,8 +301,12 @@ remove_all_move_bindings (PanelWidget *panel)
 static void
 panel_widget_class_init (PanelWidgetClass *class)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GObjectClass *object_class = (GObjectClass*) class;
+#else
 	GtkObjectClass *object_class = (GtkObjectClass*) class;
 	GObjectClass *gobject_class = (GObjectClass*) class;
+#endif
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 	GtkContainerClass *container_class = (GtkContainerClass*) class;
 
@@ -407,16 +438,31 @@ panel_widget_class_init (PanelWidgetClass *class)
 	class->tab_move = panel_widget_tab_move;
 	class->end_move = panel_widget_end_move;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	object_class->dispose = panel_widget_dispose;
+	object_class->finalize = panel_widget_finalize;
+#else
 	object_class->destroy = panel_widget_destroy;
 	gobject_class->finalize = panel_widget_finalize;
-	
+#endif
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	widget_class->get_preferred_width = panel_widget_get_preferred_width;
+	widget_class->get_preferred_height = panel_widget_get_preferred_height;
+#else
 	widget_class->size_request = panel_widget_size_request;
+#endif
 	widget_class->size_allocate = panel_widget_size_allocate;
 	widget_class->realize = panel_widget_realize;
 	widget_class->unrealize = panel_widget_unrealize;
 	widget_class->focus = panel_widget_real_focus;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	widget_class->state_flags_changed = panel_widget_state_flags_changed;
+	widget_class->style_updated = panel_widget_style_updated;
+#else
 	widget_class->state_changed = panel_widget_state_changed;
 	widget_class->style_set = panel_widget_style_set;
+#endif
 
 	container_class->add = panel_widget_cadd;
 	container_class->remove = panel_widget_cremove;
@@ -1214,7 +1260,11 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 	for(list = panel->applet_list; list!=NULL; list = g_list_next(list)) {
 		AppletData *ad = list->data;
 		GtkRequisition chreq;
-		gtk_widget_size_request(ad->applet,&chreq);
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gtk_widget_get_preferred_size (ad->applet, &chreq, NULL);
+#else
+		gtk_widget_size_request(ad->applet, &chreq);
+#endif
 
 		if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 			if (requisition->height < chreq.height && !ad->size_constrained)
@@ -1294,6 +1344,28 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 	}
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+panel_widget_get_preferred_width (GtkWidget *widget,
+								  gint *minimum_width,
+								  gint *natural_width)
+{
+	GtkRequisition req;
+	panel_widget_size_request (widget, &req);
+	*minimum_width = *natural_width = req.width;
+}
+
+static void
+panel_widget_get_preferred_height (GtkWidget *widget,
+								   gint *minimum_height,
+								   gint *natural_height)
+{
+	GtkRequisition req;
+	panel_widget_size_request (widget, &req);
+	*minimum_height = *natural_height = req.height;
+}
+#endif
+
 static void
 queue_resize_on_all_applets(PanelWidget *panel)
 {
@@ -1372,7 +1444,11 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			AppletData *ad = list->data;
 			GtkAllocation challoc;
 			GtkRequisition chreq;
-			gtk_widget_get_child_requisition(ad->applet,&chreq);
+#if GTK_CHECK_VERSION (3, 0, 0)
+			gtk_widget_get_preferred_size (ad->applet, &chreq, NULL);
+#else
+			gtk_widget_get_child_requisition(ad->applet, &chreq);
+#endif
 
 			ad->constrained = i;
 			
@@ -1423,7 +1499,11 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			AppletData *ad = list->data;
 			GtkRequisition chreq;
 
-			gtk_widget_get_child_requisition(ad->applet,&chreq);
+#if GTK_CHECK_VERSION (3, 0, 0)
+			gtk_widget_get_preferred_size (ad->applet, &chreq, NULL);
+#else
+			gtk_widget_get_child_requisition(ad->applet, &chreq);
+#endif
 
 			if (!ad->expand_major || !ad->size_hints) {
 				if(panel->orient == GTK_ORIENTATION_HORIZONTAL)
@@ -1493,7 +1573,11 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			AppletData *ad = list->data;
 			GtkAllocation challoc;
 			GtkRequisition chreq;
-			gtk_widget_get_child_requisition(ad->applet,&chreq);
+#if GTK_CHECK_VERSION (3, 0, 0)
+			gtk_widget_get_preferred_size (ad->applet, &chreq, NULL);
+#else
+			gtk_widget_get_child_requisition (ad->applet, &chreq);
+#endif
 			challoc.width = chreq.width;
 			challoc.height = chreq.height;
 			if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
@@ -1512,6 +1596,9 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				challoc.y = ad->constrained;
 			}
 			
+			challoc.width = MAX(challoc.width, 1);
+			challoc.height = MAX(challoc.height, 1);
+			
 			gtk_widget_size_allocate(ad->applet,&challoc);
 		}
 	}
@@ -1555,27 +1642,61 @@ panel_widget_is_cursor(PanelWidget *panel, int overlap)
 }
 
 static void
-panel_widget_style_set (GtkWidget *widget,
-			GtkStyle  *previous_style)
+#if GTK_CHECK_VERSION (3, 0, 0)
+panel_widget_set_background_default_style (GtkWidget *widget)
+#else
+panel_widget_style_set (GtkWidget *widget, GtkStyle  *previous_style)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *context;
+	GtkStateFlags state;
+	GdkRGBA bg_color;
+	cairo_pattern_t *bg_image;
+#else
 	GtkStyle     *style;
 	GtkStateType  state;
+#endif
 
 	if (gtk_widget_get_realized (widget)) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		context = gtk_widget_get_style_context (widget);
+		state = gtk_widget_get_state_flags (widget);
+
+		gtk_style_context_get_background_color (context, state, &bg_color);
+		gtk_style_context_get (context, state, "background-image", &bg_image, NULL);
+#else
 		style = gtk_widget_get_style (widget);
 		state = gtk_widget_get_state (widget);
+#endif
 
 		panel_background_set_default_style (
 			&PANEL_WIDGET (widget)->background,
+#if GTK_CHECK_VERSION (3, 0, 0)
+			&bg_color,
+			bg_image);
+
+			if (bg_image)
+				cairo_pattern_destroy (bg_image);
+#else
 			&style->bg [state],
 			style->bg_pixmap [state]);
+#endif
 	}
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+panel_widget_style_updated (GtkWidget *widget)
+#else
 panel_widget_state_changed (GtkWidget    *widget,
 			    GtkStateType  previous_state)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	panel_widget_set_background_default_style (widget);
+	GTK_WIDGET_CLASS (panel_widget_parent_class)->style_updated (widget);
+#else
 	GtkStyle     *style;
 	GtkStateType  state;
 
@@ -1588,8 +1709,17 @@ panel_widget_state_changed (GtkWidget    *widget,
 			&style->bg [state],
 			style->bg_pixmap [state]);
 	}
+#endif
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+panel_widget_state_flags_changed (GtkWidget *widget, GtkStateFlags previous_state)
+{
+	panel_widget_set_background_default_style (widget);
+}
+#endif
+
 static gboolean
 toplevel_configure_event (GtkWidget         *widget,
 			  GdkEventConfigure *event,
@@ -1605,8 +1735,10 @@ panel_widget_realize (GtkWidget *widget)
 {
 	PanelWidget  *panel = (PanelWidget *) widget;
 	GdkWindow    *window;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyle     *style;
 	GtkStateType  state;
+#endif
 
 	g_signal_connect (panel->toplevel, "configure-event",
 			  G_CALLBACK (toplevel_configure_event), panel);
@@ -1614,17 +1746,23 @@ panel_widget_realize (GtkWidget *widget)
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->realize (widget);
 
 	window = gtk_widget_get_window (widget);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	style = gtk_widget_get_style (widget);
 	state = gtk_widget_get_state (widget);
+#endif
 
 	/* For auto-hidden panels with a colored background, we need native
 	 * windows to avoid some uglyness on unhide */
 	gdk_window_ensure_native (window);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	panel_widget_set_background_default_style (widget);
+#else
 	panel_background_set_default_style (
 		&panel->background,
 		&style->bg [state],
 		style->bg_pixmap [state]);
+#endif
 
 	panel_background_realized (&panel->background, window);
 }
@@ -1694,7 +1832,11 @@ panel_widget_destroy_open_dialogs (PanelWidget *panel_widget)
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+panel_widget_dispose (GObject *obj)
+#else
 panel_widget_destroy (GtkObject *obj)
+#endif
 {
 	PanelWidget *panel;
 
@@ -1715,8 +1857,12 @@ panel_widget_destroy (GtkObject *obj)
 		panel->master_widget = NULL;
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	G_OBJECT_CLASS (panel_widget_parent_class)->dispose (obj);
+#else
 	if (GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy)
 		GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy (obj);
+#endif
 }
 
 static void
@@ -2328,8 +2474,12 @@ panel_widget_applet_key_press_event (GtkWidget   *widget,
 
 	if (!mate_panel_applet_in_drag)
 		return FALSE;
-	
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	return gtk_bindings_activate (G_OBJECT (panel),
+#else
 	return gtk_bindings_activate (GTK_OBJECT (panel),
+#endif
 				      ((GdkEventKey *)event)->keyval, 
 				      ((GdkEventKey *)event)->state);	
 }
