@@ -27,6 +27,7 @@
 #include "panel-xutils.h"
 
 #include <glib.h>
+#include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
@@ -50,7 +51,11 @@ panel_xutils_set_window_type (GdkWindow             *gdk_window,
 	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
 
 	display = GDK_WINDOW_XDISPLAY (gdk_window);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	window = GDK_WINDOW_XID (gdk_window);
+#else
 	window  = GDK_WINDOW_XWINDOW (gdk_window);
+#endif
 
 	if (net_wm_window_type == None)
 		net_wm_window_type = XInternAtom (display,
@@ -81,7 +86,11 @@ panel_xutils_set_window_type (GdkWindow             *gdk_window,
 	XChangeProperty (display, window, net_wm_window_type,
 			 XA_ATOM, 32, PropModeReplace,
 			 (guchar *) &atoms, i);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+#else
 	gdk_error_trap_pop ();
+#endif
 }
 
 enum {
@@ -113,7 +122,11 @@ panel_xutils_set_strut (GdkWindow        *gdk_window,
 	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
 
 	display = GDK_WINDOW_XDISPLAY (gdk_window);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	window = GDK_WINDOW_XID (gdk_window);
+#else
 	window  = GDK_WINDOW_XWINDOW (gdk_window);
+#endif
 
 	if (net_wm_strut == None)
 		net_wm_strut = XInternAtom (display, "_NET_WM_STRUT", False);
@@ -150,7 +163,11 @@ panel_xutils_set_strut (GdkWindow        *gdk_window,
 	XChangeProperty (display, window, net_wm_strut_partial,
 			 XA_CARDINAL, 32, PropModeReplace,
 			 (guchar *) &struts, 12);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+#else
 	gdk_error_trap_pop ();
+#endif
 }
 
 void
@@ -164,11 +181,19 @@ panel_warp_pointer (GdkWindow *gdk_window,
 	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
 
 	display = GDK_WINDOW_XDISPLAY (gdk_window);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	window = GDK_WINDOW_XID (gdk_window);
+#else
 	window  = GDK_WINDOW_XWINDOW (gdk_window);
+#endif
 
 	gdk_error_trap_push ();
 	XWarpPointer (display, None, window, 0, 0, 0, 0, x, y);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+#else
 	gdk_error_trap_pop ();
+#endif
 }
 
 guint
